@@ -9,6 +9,7 @@ from sqlalchemy.inspection import inspect
 from sqlalchemy.orm import joinedload, selectinload
 from sqlalchemy.orm.attributes import InstrumentedAttribute
 from sqlalchemy.orm.collections import InstrumentedList
+from sqlalchemy.sql import column, distinct
 
 from fastapi_jsonapi import BadRequest
 from fastapi_jsonapi.data_layers.base import BaseDataLayer
@@ -115,6 +116,7 @@ class SqlalchemyDataLayer(BaseDataLayer):
     def prepare_id_value(self, col: InstrumentedAttribute, value: Any) -> Any:
         """
         Convert value to the required python type.
+
         Type is declared on the SQLA column.
 
         :param col:
@@ -184,27 +186,6 @@ class SqlalchemyDataLayer(BaseDataLayer):
             # todo: relation name may be different?
             setattr(obj, relation_name, related_data)
 
-    def _apply_client_generated_id(
-        self,
-        data_create: BaseJSONAPIItemInSchema,
-        model_kwargs: dict,
-    ):
-        """
-        :param data_create: the data validated by pydantic.
-        :param model_kwargs: the data validated by pydantic.
-        """
-        if data_create.id is None:
-            return model_kwargs
-
-        extra = data_create.__fields__["id"].field_info.extra
-        if extra.get("client_can_set_id"):
-            id_value = data_create.id
-            if cast_type := extra.get("id_cast_func"):
-                id_value = cast_type(id_value)
-            model_kwargs["id"] = id_value
-
-        return model_kwargs
-
     async def create_object(self, data_create: BaseJSONAPIItemInSchema, view_kwargs: dict) -> TypeModel:
         """
         Create an object through sqlalchemy.
@@ -246,6 +227,7 @@ class SqlalchemyDataLayer(BaseDataLayer):
     def get_object_id_field_name(self):
         """
         compound key may cause errors
+
         :return:
         """
         return self.id_name_field or inspect(self.model).primary_key[0].key
@@ -283,6 +265,8 @@ class SqlalchemyDataLayer(BaseDataLayer):
 
     async def get_collection_count(self, query: "Select", qs: QueryStringManager, view_kwargs: dict) -> int:
         """
+        Returns number of elements for this collection
+
         :param query: SQLAlchemy query
         :param qs: QueryString
         :param view_kwargs: view kwargs
@@ -291,7 +275,8 @@ class SqlalchemyDataLayer(BaseDataLayer):
         if self.disable_collection_count is True:
             return self.default_collection_count
 
-        return (await self.session.execute(select(func.count()).select_from(query.subquery()))).scalar_one()
+        count_query = select(func.count(distinct(column("id")))).select_from(query.subquery())
+        return (await self.session.execute(count_query)).scalar_one()
 
     async def get_collection(self, qs: QueryStringManager, view_kwargs: Optional[dict] = None) -> Tuple[int, list]:
         """
@@ -320,7 +305,7 @@ class SqlalchemyDataLayer(BaseDataLayer):
 
         query = self.paginate_query(query, qs.pagination)
 
-        collection = (await self.session.execute(query)).scalars().all()
+        collection = (await self.session.execute(query)).unique().scalars().all()
 
         collection = await self.after_get_collection(collection, qs, view_kwargs)
 
@@ -537,6 +522,40 @@ class SqlalchemyDataLayer(BaseDataLayer):
         :param view_kwargs: kwargs from the resource view.
         """
 
+    def get_related_model_query_base(
+        self,
+        related_model: Type[TypeModel],
+    ) -> "Select":
+        """
+        Prepare sql query (statement) to fetch related model
+
+        :param related_model:
+        :return:
+        """
+        return select(related_model)
+
+    def get_related_object_query(
+        self,
+        related_model: Type[TypeModel],
+        related_id_field: str,
+        id_value: str,
+    ):
+        id_field = getattr(related_model, related_id_field)
+        id_value = self.prepare_id_value(id_field, id_value)
+        stmt: "Select" = self.get_related_model_query_base(related_model)
+        return stmt.where(id_field == id_value)
+
+    def get_related_objects_list_query(
+        self,
+        related_model: Type[TypeModel],
+        related_id_field: str,
+        ids: list[str],
+    ) -> Tuple["Select", list[str]]:
+        id_field = getattr(related_model, related_id_field)
+        prepared_ids = [self.prepare_id_value(id_field, _id) for _id in ids]
+        stmt: "Select" = self.get_related_model_query_base(related_model)
+        return stmt.where(id_field.in_(prepared_ids)), prepared_ids
+
     async def get_related_object(
         self,
         related_model: Type[TypeModel],
@@ -551,9 +570,12 @@ class SqlalchemyDataLayer(BaseDataLayer):
         :param id_value: related object id value
         :return: a related SQLA ORM object
         """
-        id_field = getattr(related_model, related_id_field)
-        id_value = self.prepare_id_value(id_field, id_value)
-        stmt = select(related_model).where(id_field == id_value)
+        stmt = self.get_related_object_query(
+            related_model=related_model,
+            related_id_field=related_id_field,
+            id_value=id_value,
+        )
+
         try:
             related_object = (await self.session.execute(stmt)).scalar_one()
         except NoResultFound:
@@ -569,15 +591,18 @@ class SqlalchemyDataLayer(BaseDataLayer):
         ids: list[str],
     ) -> list[TypeModel]:
         """
+        Fetch related objects (many)
 
         :param related_model:
         :param related_id_field:
         :param ids:
         :return:
         """
-        id_field = getattr(related_model, related_id_field)
-        ids = [self.prepare_id_value(id_field, _id) for _id in ids]
-        stmt = select(related_model).where(id_field.in_(ids))
+        stmt, ids = self.get_related_objects_list_query(
+            related_model=related_model,
+            related_id_field=related_id_field,
+            ids=ids,
+        )
 
         related_objects = (await self.session.execute(stmt)).scalars().all()
         object_ids = [getattr(obj, related_id_field) for obj in related_objects]
